package cat.bcn.noise;
import java.util.Calendar;
import java.util.List;
import android.app.Activity;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
/**
 * Proporciona la localizacion geografica del disponsitivo en cada momento
 * Para ello utiliza tres proveedores: GPS, puntos WIFI y antenas de telefonía
 * @author Esteban Roman Castellanos
 * @version 1.0
*/ 
//public class GeoLocation extends Activity {
public class GeoLocation {
	private double longitud = 0;
	private double latitud = 0; 
	private double precision = 0;
	private long desfase;
	private float velocidad;
	private String proveedor; 
	private int numeroProveedores = 0;
	private LocationListener locListener;
	private Location loc;
	private boolean isGPSEnabled;
	private boolean isNetworkEnabled;
	private List<String> listaProviders;
	private Calendar ca = Calendar.getInstance();
	public GeoLocation (final LocationManager locManager)
	{
		listaProviders = locManager.getAllProviders();
		numeroProveedores = listaProviders.size();
		isConnect(locManager);
			if(!isNetworkEnabled && !isGPSEnabled)
				{
				proveedor = "off";
				}else{
				//suscripción a los cambios del proveedor
				locListener = new LocationListener() {
					public void onLocationChanged(Location loc){
						if(loc != null){
							fixLocation();
						};
					}
					public void onProviderDisabled(String provider){
						isConnect(locManager);
						//fixLocation();
					}
					public void onProviderEnabled(String provider){
						isConnect(locManager);
						//fixLocation();
					}
					public void onStatusChanged(String provider, int status, Bundle extras){
						isConnect(locManager);
						//fixLocation();
					}
				};
			/**
			 * El LocationListener interroga al proveedor cada 3 segundos 
			 * Es una Suscripción a los dos proveedores. 
			 * El listener correspondiente al network no funciona. 
			 */
	    	locManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 3000, 0, locListener);
	    	locManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 3000, 0, locListener);
			}
	}
	/**
	 * Devuelve la latitud. 
	 * @return latitud
	 */
	public double getLatitud() {
		return latitud;
    }
	/**
	 * Devuelve la Lontitud de la posición 
	 * @return longitud
	 */
	public double getLongitud() {
		return longitud;
    }
	/**
	 * Devuelve la precisión de la localización geográfica
	 * @return precision
	 */
	public double getPrecision(){
		return precision;
	}
	/**
	 * Nos devuelve el proveedor de geolocalización que estamos usando en cada momento
	 * @return proveedor
	 */
	public String getProveedor(){
		return proveedor; 
	}
	/**
	 * Nos devuelve el tiempo transcurrido desde que se tomó la observación. 
	 * @return proveedor
	 */
	public long getTiempo(){
		return desfase;
	}
	/**
	 * Nos muestra el número de proveedores de localización en el movil aunque no estén activos. 
	 * @return
	 */
	public int getNumeroProveedores(){
		return numeroProveedores;
	}
	/**
	 * Nos da el mejor proveedor utilizando como métrica la precisión
	 * @return mejorProviderCrit
	 */
	public List<String> getProveedores(){
		return listaProviders;
	}
	public float getVelocidad(){
		return velocidad;
	}
	/**
	 * Actualiza el valor de la longitud, latitud, precisión, etc. 
	 * Además calcula el tiempo que ha pasado desde que se tomo la observación.
	 * Si Network no está permitido y GPS está en proceso de obtener observación loc = null
	 */
	private void fixLocation(){
		if (loc != null){
			
			longitud = loc.getLongitude();
			latitud = loc.getLatitude();
			precision = loc.getAccuracy();
			proveedor = loc.getProvider(); 
			velocidad =loc.getSpeed();
			desfase = (ca.getTimeInMillis() - loc.getTime())/1000;
		}
		
	}
	/**
	 * Para el Listener sobre el GPS
	 */
	public void stopListener(LocationManager locManager){
		if(locListener != null){
		locManager.removeUpdates(locListener);}
	}
	private void isConnect(LocationManager locManager){
		isGPSEnabled = locManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
		isNetworkEnabled = locManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
		if(isGPSEnabled){
			if (loc != null){
				loc = null;
			}
			loc = locManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
			   /**
			    * El GPS puede estar activo pero no tener todavía una localización
			    */
			if(loc != null){
				   /**
				    * Las localizaciones más antiguas de 30 segundos se desechan. 
				    */
				desfase = (ca.getTimeInMillis() - loc.getTime())/1000;
				if(desfase > 30){
					loc = null;
				}else{
				fixLocation();
				//Ponemos a false el NETWORK para que aunque esté activo deje el GPS.
				isNetworkEnabled = false;}
			};
		}
		if(isNetworkEnabled){
			if (loc != null){
				loc = null;
			}
			loc = locManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);
			if(loc != null){
				fixLocation();
			};
		}
	}
}