import java.io.*;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.Set;
import java.io.PrintWriter;
import java.net.MalformedURLException;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import redis.clients.jedis.*;
import redis.clients.util.*;
import redis.clients.jedis.exceptions.*;

/**
 * Servlet para implementar el servicio 
 * @author Esteban Roman Castellanos
 */
@WebServlet("/Sensor")
public class Sensor extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public Sensor() {
        super();
        // TODO Auto-generated constructor stub
    }

	/**
	 * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException, 
		NumberFormatException, JedisConnectionException, JedisDataException, MalformedURLException {
		// TODO Auto-generated method stub
	    try{
	    	long horas;
	    	horas = Long.parseLong(request.getParameter("horas"));
		//String sensor = request.getParameter("sensor");
		//Si ponemos sensor como entero podemos validar la correcta codificación
		//Después parece que no da problemas al inclirlo en el string que mandamos a redis  	
		int sensor = Integer.parseInt(request.getParameter("sensor"));
		if(horas < 0 || sensor < 0){throw new NumberFormatException();}
		Jedis jedis = new Jedis("www.peripateticos.es");
		jedis.auth("sentilo");
		jedis.dbSize();
		String json = "";
		String location;
		String tiempo;
		String data;
		String strLatitud;
		float latitud;
		String strLongitud;
		float longitud;
		int blanco;
		int i = 0;
	    Calendar hoy = Calendar.getInstance();
	    long ahora = hoy.getTimeInMillis();
	    long inicio = ahora - 3600*horas*1000;
	    String clave = "sid:" + sensor + ":observations";
		Set<String> sdid = jedis.zrangeByScore(clave, inicio, ahora);
		if(sdid.isEmpty()){throw new NullPointerException();}
		Iterator it=sdid.iterator();
		JSONObject obj = new JSONObject();
		JSONArray lisPuntos = new JSONArray();
		while(it.hasNext())
	        {
		  JSONObject puntos = new JSONObject();
		  JSONObject punto = new JSONObject();
		  JSONObject valores = new JSONObject();
		  JSONArray coordenadas = new JSONArray();
		  String iteracion = (String)it.next();
	      String value ="sdid:" + iteracion; 
	      location = jedis.hget(value,"location");
		  tiempo = jedis.hget(value,"ts");
		  data = jedis.hget(value, "data");
		  if(location != null){
			  blanco = location.indexOf(" ");
			  strLatitud = location.substring(0, blanco);
			  latitud = Float.parseFloat(strLatitud);
			  strLongitud = location.substring(blanco + 1);
			  longitud = Float.parseFloat(strLongitud);
		  }else{
			 longitud = 0;
			 latitud =0;
		  }
		  coordenadas.add(longitud);
		  coordenadas.add(latitud);
		  punto.put("type", "Point");
		  valores.put("prop0",data);
		  punto.put("coordinates", coordenadas);
		  puntos.put("properties", valores);
		  puntos.put("type", "Feature");
		  puntos.put("geometry", punto);
		  lisPuntos.add(puntos);
	        }
	        obj.put("type","FeatureCollection");
		obj.put("features", lisPuntos);	
		String objstring = obj.toString(); 
		response.setHeader("Access-Control-Allow-Origin", "*");
		response.setHeader("Access-Control-Request-Method", "GET,POST");
		PrintWriter out = response.getWriter();
		out.println(objstring);
	    }catch(MalformedURLException  ejs)
	    {
	    	response.setContentType("application/json");
    		PrintWriter out = response.getWriter();
    		out.write("{ERROR:\"" + ejs.toString() + "\" ,COMENTARIO:\"" + "La URL enviada no es correcta\"}");	
	    }catch (NumberFormatException e){
    		//response.setContentType("text/html");
	    	//response.setContentType("application/json");
    		//PrintWriter out = response.getWriter();
    		if (e.getMessage() == "null"){
    			response.sendError(response.SC_BAD_REQUEST, "El sensor y/o las horas son nulos");
    			//out.write("{ERROR:\"" + e.toString() + "\" ,COMENTARIO:\"" + "El sensor o las horas son nulos\"}");
    		}else{
    			response.sendError(response.SC_BAD_REQUEST, "El sensor o las horas enviadas no son enteros positivos");
    			//out.write("{ERROR:\"" + e.toString() + "\" ,COMENTARIO:\"" + "El tipo de dato del sensor o las horas no es entero\"}");
    		}
	    }catch(JedisDataException ej)
	    {
	    	//response.setContentType("text/html");
	    	//response.setContentType("application/json");
	    	response.sendError(response.SC_UNAUTHORIZED, "Contactar con el Administrador");
    		//PrintWriter out = response.getWriter();
    		//out.write("{ERROR:\"" + ej.toString() + "\" ,COMENTARIO:\"" + "Problema con la autenticación: Contactar con el adminsitrador\"}");
    	}catch(JedisConnectionException ejs)
	    {
	    	//response.setContentType("application/json");
	    	response.sendError(response.SC_SERVICE_UNAVAILABLE, "Contactar con el Administrador");
    		//PrintWriter out = response.getWriter();
    		//out.write("{ERROR:\"" + ejs.toString() + "\" ,COMENTARIO:\"" + "El gestor REDIS no responde: Contactar con el adminsitrador\"}");
	    }catch(NullPointerException en)
	    {
	    	//response.setContentType("application/json");
	    	response.sendError(response.SC_NO_CONTENT, "Con estos parámetros no se obtiene ninguna observación");
    		//PrintWriter out = response.getWriter();
    		//out.write("{ERROR:\"" + ejs.toString() + "\" ,COMENTARIO:\"" + "El gestor REDIS no responde: Contactar con el adminsitrador\"}");
	    }
	    catch(IOException es){
	    	response.setContentType("text/html");
    		PrintWriter out = response.getWriter();
    		out.write("<html><head><title>Exception/Error Details</title></head><body>");
    		out.write("<h3>Error Details</h3>");
    		if (es.getMessage() == "null"){
    			out.write("<strong>Excepción general</strong>:"+"<br>");
    			out.write("<strong>Mensaje completo</strong>:"+ es.toString() + "<br>");
    		}else{
            out.write("<strong>Status Code</strong>:"+ es.getMessage() + "<br>");
            out.write("<strong>Status Code</strong>:"+ es.getLocalizedMessage() + "<br>");
            out.write("<strong>Mensaje completo</strong>:"+ es.toString() + "<br>");
            out.write("<strong>CONTACTAR CON EL SERVIDOR</strong>:");
            out.write("<br><br>");
            out.write("</body></html>");}	
	    	}
	}

	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		// TODO Auto-generated method stub
	}
	//private void procesarError(Object e, HttpServletResponse response){
	//	response.setContentType("application/json");
	//	PrintWriter out = response.getWriter();
	//	if (e.getMessage() == "null"){
	//		out.write("{error:\"" + e.toString() + "\" ,comentario:\"" + "El sensor o las horas son nulos\"}");
	//	}else{
	//		out.write("{error:\"" + e.toString() + "\" ,comentario:\"" + "El tipo de dato del sensor o las horas no es entero\"}");
	//	}
	//}
}
