package cat.bcn.speed;
import java.util.Calendar;
import java.util.List;
import android.app.Activity;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
/**
 * Proporciona la localizacion geografica del disponsitivo en cada momento
 * Para ello utiliza solo el GPS
 * @author Esteban Roman Castellanos
 * @version 1.0
*/ 
//public class GeoLocation extends Activity {
public class GeoLocation {
	private double longitud = 0;
	private double latitud = 0; 
	private double precision = 0;
	private float velocidad;
	private long desfase;
	private String proveedor; 
	private LocationListener locListener;
	private Location loc;
	private boolean isGPSEnabled;
	private Calendar ca = Calendar.getInstance();
	public GeoLocation (final LocationManager locManager)
	{
		isGPSEnabled = locManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
			if(!isGPSEnabled)
				{
				proveedor = "nulo";
				}else{
				//suscripción a los cambios del proveedor
					loc = locManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
					if(loc != null){
						desfase = (ca.getTimeInMillis() - loc.getTime())/1000;
						if(desfase > 30)
						{
							loc = null;
							proveedor = "sin datos";
						}else{
						fixLocation();
						}
					}else{
						proveedor = "sin datos";
					}	
					
				locListener = new LocationListener() {
					public void onLocationChanged(Location loc){
						if(loc != null){
							fixLocation();
						};
					}
					public void onProviderDisabled(String provider){
						loc = null;
					}
					public void onProviderEnabled(String provider){
						loc = locManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
					}
					public void onStatusChanged(String provider, int status, Bundle extras){
					}
				};
			/**
			 * El LocationListener interroga al proveedor cada 3 segundos 
			 */
	    	locManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 3000, 0, locListener);
			}
	}
	/**
	 * Devuelve la latitud. 
	 * @return latitud
	 */
	public double getLatitud() {
		return latitud;
    }
	/**
	 * Devuelve la Lontitud de la posición 
	 * @return longitud
	 */
	public double getLongitud() {
		return longitud;
    }
	/**
	 * Devuelve la precisión de la localización geográfica
	 * @return precision
	 */
	public double getPrecision(){
		return precision;
	}
	/**
	 * Nos devuelve el proveedor de geolocalización que estamos usando en cada momento
	 * @return proveedor
	 */
	public String getProveedor(){
		return proveedor; 
	}
	/**
	 * Nos devuelve el tiempo transcurrido desde que se tomó la observación. 
	 * @return proveedor
	 */
	public long getTiempo(){
		return desfase;
	}
	/**
	 * Cuando está disponible nos devuelve una medida de la velocidad de desplazamiento.  
	 * @return
	 */
	public float getVelocidad(){
		return velocidad;
	}
	/**
	 * Actualiza el valor de la longitud, latitud, precisión, etc. 
	 * Además calcula el tiempo que ha pasado desde que se tomo la observación.
	 * Si Network no está permitido y GPS está en proceso de obtener observación loc = null
	 */
	private void fixLocation(){
		if (loc != null){
			longitud = loc.getLongitude();
			latitud = loc.getLatitude();
			precision = loc.getAccuracy();
			//if (loc.getProvider() != null){
			//	proveedor = "gps";	
			//}
			proveedor = loc.getProvider();
			if(loc.hasSpeed()){
				velocidad = loc.getSpeed() * (float)3.6;
			}else{
				velocidad = -1;
			}
		}
	}
	/**
	 * Para el Listener sobre el GPS
	 */
	public void stopListener(LocationManager locManager){
		if(locListener != null){
		locManager.removeUpdates(locListener);}
	}
}